"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var template_1 = require("../template");
var types_1 = require("../types");
var literals_1 = require("./literals");
var typeconvert_1 = require("./typeconvert");
var CElementAccess = /** @class */ (function () {
    function CElementAccess(scope, node) {
        var type = null;
        var elementAccess = null;
        var argumentExpression = null;
        if (ts.isIdentifier(node)) {
            type = scope.root.typeHelper.getCType(node);
            elementAccess = node.text;
            var isLogicalContext = (node.parent.kind == ts.SyntaxKind.IfStatement
                || node.parent.kind == ts.SyntaxKind.WhileStatement
                || node.parent.kind == ts.SyntaxKind.DoStatement) && node.parent["expression"] == node;
            if (!isLogicalContext && node.parent.kind == ts.SyntaxKind.ForStatement && node.parent["condition"] == node)
                isLogicalContext = true;
            if (!isLogicalContext && node.parent.kind == ts.SyntaxKind.BinaryExpression) {
                var binExpr = node.parent;
                if (binExpr.operatorToken.kind == ts.SyntaxKind.AmpersandAmpersandToken
                    || binExpr.operatorToken.kind == ts.SyntaxKind.BarBarToken)
                    isLogicalContext = true;
            }
            if (!isLogicalContext && node.parent.kind == ts.SyntaxKind.PrefixUnaryExpression) {
                var binExpr = node.parent;
                if (binExpr.operator == ts.SyntaxKind.ExclamationToken)
                    isLogicalContext = true;
            }
            if (isLogicalContext && type instanceof types_1.ArrayType && !type.isDynamicArray) {
                argumentExpression = "0";
            }
        }
        else if (node.kind == ts.SyntaxKind.PropertyAccessExpression) {
            var propAccess = node;
            type = scope.root.typeHelper.getCType(propAccess.expression);
            if (ts.isIdentifier(propAccess.expression))
                elementAccess = propAccess.expression.text;
            else
                elementAccess = new CElementAccess_1(scope, propAccess.expression);
            if (type === types_1.UniversalVarType) {
                argumentExpression = 'js_var_from_str("' + propAccess.name.text + '")';
                scope.root.headerFlags.js_var_from_str = true;
            }
            else if (type instanceof types_1.DictType)
                argumentExpression = '"' + propAccess.name.text + '"';
            else
                argumentExpression = propAccess.name.text;
        }
        else if (node.kind == ts.SyntaxKind.ElementAccessExpression) {
            var elemAccess = node;
            type = scope.root.typeHelper.getCType(elemAccess.expression);
            if (ts.isIdentifier(elemAccess.expression))
                elementAccess = elemAccess.expression.text;
            else
                elementAccess = new CElementAccess_1(scope, elemAccess.expression);
            if (type === types_1.UniversalVarType)
                argumentExpression = new typeconvert_1.CAsUniversalVar(scope, elemAccess.argumentExpression);
            else if (type instanceof types_1.StructType && elemAccess.argumentExpression.kind == ts.SyntaxKind.StringLiteral) {
                var ident = elemAccess.argumentExpression.getText().slice(1, -1);
                if (ident.search(/^[_A-Za-z][_A-Za-z0-9]*$/) > -1)
                    argumentExpression = ident;
                else
                    argumentExpression = template_1.CodeTemplateFactory.createForNode(scope, elemAccess.argumentExpression);
            }
            else
                argumentExpression = template_1.CodeTemplateFactory.createForNode(scope, elemAccess.argumentExpression);
        }
        else {
            type = scope.root.typeHelper.getCType(node);
            elementAccess = template_1.CodeTemplateFactory.createForNode(scope, node);
        }
        this.simpleAccessor = new CSimpleElementAccess(scope, type, elementAccess, argumentExpression);
    }
    CElementAccess_1 = CElementAccess;
    CElementAccess = CElementAccess_1 = __decorate([
        template_1.CodeTemplate("{simpleAccessor}", [ts.SyntaxKind.ElementAccessExpression, ts.SyntaxKind.PropertyAccessExpression, ts.SyntaxKind.Identifier])
    ], CElementAccess);
    return CElementAccess;
    var CElementAccess_1;
}());
exports.CElementAccess = CElementAccess;
var CSimpleElementAccess = /** @class */ (function () {
    function CSimpleElementAccess(scope, type, elementAccess, argumentExpression) {
        this.elementAccess = elementAccess;
        this.argumentExpression = argumentExpression;
        this.isDynamicArray = false;
        this.isStaticArray = false;
        this.isStruct = false;
        this.isDict = false;
        this.isString = false;
        this.nullValue = "0";
        this.isUniversalAccess = false;
        this.isSimpleVar = typeof type === 'string' && type != types_1.UniversalVarType && type != types_1.PointerVarType;
        this.isDynamicArray = type instanceof types_1.ArrayType && type.isDynamicArray;
        this.isStaticArray = type instanceof types_1.ArrayType && !type.isDynamicArray;
        this.arrayCapacity = type instanceof types_1.ArrayType && !type.isDynamicArray && type.capacity + "";
        this.isDict = type instanceof types_1.DictType;
        this.isStruct = type instanceof types_1.StructType;
        if (type === types_1.UniversalVarType && argumentExpression != null) {
            this.isUniversalAccess = true;
            scope.root.headerFlags.js_var_get = true;
        }
        this.isString = type === types_1.StringVarType;
        if (argumentExpression != null && type instanceof types_1.DictType && type.elementType === types_1.UniversalVarType)
            this.nullValue = new literals_1.CUndefined(scope);
        if (this.isString && this.argumentExpression == "length")
            scope.root.headerFlags.str_len = true;
    }
    CSimpleElementAccess = __decorate([
        template_1.CodeTemplate("\n{#if isString && argumentExpression == 'length'}\n    str_len({elementAccess})\n{#elseif isSimpleVar || argumentExpression == null}\n    {elementAccess}\n{#elseif isDynamicArray && argumentExpression == 'length'}\n    {elementAccess}->size\n{#elseif isDynamicArray}\n    {elementAccess}->data[{argumentExpression}]\n{#elseif isStaticArray && argumentExpression == 'length'}\n    {arrayCapacity}\n{#elseif isStaticArray}\n    {elementAccess}[{argumentExpression}]\n{#elseif isStruct}\n    {elementAccess}->{argumentExpression}\n{#elseif isDict}\n    DICT_GET({elementAccess}, {argumentExpression}, {nullValue})\n{#elseif isUniversalAccess}\n    js_var_get({elementAccess}, {argumentExpression})\n{#else}\n    /* Unsupported element access scenario: {elementAccess} {argumentExpression} */\n{/if}")
    ], CSimpleElementAccess);
    return CSimpleElementAccess;
}());
exports.CSimpleElementAccess = CSimpleElementAccess;
var CArraySize = /** @class */ (function () {
    function CArraySize(scope, varAccess, type) {
        this.varAccess = varAccess;
        this.type = type;
        this.arrayCapacity = type.capacity + "";
    }
    CArraySize = __decorate([
        template_1.CodeTemplate("\n{#if type.isDynamicArray}\n    {varAccess}->size\n{#else}\n    {arrayCapacity}\n{/if}")
    ], CArraySize);
    return CArraySize;
}());
exports.CArraySize = CArraySize;
//# sourceMappingURL=elementaccess.js.map