"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var assignment_1 = require("./assignment");
var ts = require("typescript");
var template_1 = require("../template");
var types_1 = require("../types");
var variable_1 = require("./variable");
var regexfunc_1 = require("./regexfunc");
var literals_1 = require("./literals");
var typeconvert_1 = require("./typeconvert");
var typeguards_1 = require("../typeguards");
var CCondition = /** @class */ (function () {
    function CCondition(scope, node) {
        this.universalWrapper = false;
        this.isString = false;
        this.expressionIsIdentifier = false;
        this.expression = template_1.CodeTemplateFactory.createForNode(scope, node);
        this.expressionIsIdentifier = ts.isIdentifier(node);
        var type = scope.root.typeHelper.getCType(node);
        this.isString = type == types_1.StringVarType;
        if (type == types_1.UniversalVarType) {
            this.universalWrapper = true;
            scope.root.headerFlags.js_var_to_bool = true;
        }
    }
    CCondition = __decorate([
        template_1.CodeTemplate("\n{#if universalWrapper}\n    js_var_to_bool({expression})\n{#elseif isString && expressionIsIdentifier}\n    *{expression}\n{#elseif isString}\n    *({expression})\n{#else}\n    {expression}\n{/if}")
    ], CCondition);
    return CCondition;
}());
exports.CCondition = CCondition;
var CBinaryExpression = /** @class */ (function () {
    function CBinaryExpression(scope, node) {
        this.replacedWithCall = false;
        this.callAddArgs = "";
        this.expression = null;
        if (node.operatorToken.kind == ts.SyntaxKind.EqualsToken) {
            this.expression = assignment_1.AssignmentHelper.create(scope, node.left, node.right, true);
            return;
        }
        if (node.operatorToken.kind == ts.SyntaxKind.CommaToken) {
            var nodeAsStatement = ts.createNode(ts.SyntaxKind.ExpressionStatement);
            nodeAsStatement.expression = node.left;
            nodeAsStatement.parent = node.getSourceFile();
            scope.statements.push(template_1.CodeTemplateFactory.createForNode(scope, nodeAsStatement));
            this.expression = template_1.CodeTemplateFactory.createForNode(scope, node.right);
            return;
        }
        if (node.operatorToken.kind == ts.SyntaxKind.PlusToken) {
            this.expression = new CPlusExpression(scope, node);
            return;
        }
        if (node.operatorToken.kind == ts.SyntaxKind.PlusEqualsToken) {
            var left = template_1.CodeTemplateFactory.createForNode(scope, node.left);
            var right = new CPlusExpression(scope, node);
            this.expression = "(" + template_1.CodeTemplateFactory.templateToString(left) + " = " + template_1.CodeTemplateFactory.templateToString(right) + ")";
        }
        if (typeguards_1.isNumberOp(node.operatorToken.kind) || typeguards_1.isIntegerOp(node.operatorToken.kind)) {
            this.expression = new CArithmeticExpression(scope, node);
            return;
        }
        var leftType = scope.root.typeHelper.getCType(node.left);
        this.left = template_1.CodeTemplateFactory.createForNode(scope, node.left);
        var rightType = scope.root.typeHelper.getCType(node.right);
        this.right = template_1.CodeTemplateFactory.createForNode(scope, node.right);
        var operatorKind = node.operatorToken.kind;
        var operatorMap = {};
        var callReplaceMap = {};
        operatorMap[ts.SyntaxKind.AmpersandAmpersandToken] = '&&';
        operatorMap[ts.SyntaxKind.BarBarToken] = '||';
        var isEqualityOp = types_1.equalityOps.indexOf(operatorKind) > -1;
        if (leftType == types_1.BooleanVarType && types_1.operandsToNumber(leftType, operatorKind, rightType))
            leftType = types_1.NumberVarType;
        if (rightType == types_1.BooleanVarType && types_1.operandsToNumber(leftType, operatorKind, rightType))
            rightType = types_1.NumberVarType;
        if (leftType == types_1.NumberVarType && rightType == types_1.NumberVarType) {
            operatorMap[ts.SyntaxKind.GreaterThanToken] = '>';
            operatorMap[ts.SyntaxKind.GreaterThanEqualsToken] = '>=';
            operatorMap[ts.SyntaxKind.LessThanToken] = '<';
            operatorMap[ts.SyntaxKind.LessThanEqualsToken] = '<=';
            operatorMap[ts.SyntaxKind.ExclamationEqualsEqualsToken] = '!=';
            operatorMap[ts.SyntaxKind.ExclamationEqualsToken] = '!=';
            operatorMap[ts.SyntaxKind.EqualsEqualsEqualsToken] = '==';
            operatorMap[ts.SyntaxKind.EqualsEqualsToken] = '==';
        }
        else if (leftType == types_1.StringVarType && rightType == types_1.StringVarType) {
            callReplaceMap[ts.SyntaxKind.ExclamationEqualsEqualsToken] = ['strcmp', ' != 0'];
            callReplaceMap[ts.SyntaxKind.ExclamationEqualsToken] = ['strcmp', ' != 0'];
            callReplaceMap[ts.SyntaxKind.EqualsEqualsEqualsToken] = ['strcmp', ' == 0'];
            callReplaceMap[ts.SyntaxKind.EqualsEqualsToken] = ['strcmp', ' == 0'];
            if (callReplaceMap[operatorKind])
                scope.root.headerFlags.strings = true;
        }
        else if (leftType == types_1.NumberVarType && rightType == types_1.StringVarType
            || leftType == types_1.StringVarType && rightType == types_1.NumberVarType) {
            callReplaceMap[ts.SyntaxKind.ExclamationEqualsToken] = ['str_int16_t_cmp', ' != 0'];
            callReplaceMap[ts.SyntaxKind.EqualsEqualsToken] = ['str_int16_t_cmp', ' == 0'];
            if (callReplaceMap[operatorKind]) {
                scope.root.headerFlags.str_int16_t_cmp = true;
                // str_int16_t_cmp expects certain order of arguments (string, number)
                if (leftType == types_1.NumberVarType) {
                    var tmp = this.left;
                    this.left = this.right;
                    this.right = tmp;
                }
            }
        }
        else if (isEqualityOp && (leftType == types_1.UniversalVarType || rightType == types_1.UniversalVarType)) {
            callReplaceMap[ts.SyntaxKind.ExclamationEqualsEqualsToken] = ['js_var_eq|, TRUE', ' == FALSE'];
            callReplaceMap[ts.SyntaxKind.ExclamationEqualsToken] = ['js_var_eq|, FALSE', ' == FALSE'];
            callReplaceMap[ts.SyntaxKind.EqualsEqualsEqualsToken] = ['js_var_eq|, TRUE', ' == TRUE'];
            callReplaceMap[ts.SyntaxKind.EqualsEqualsToken] = ['js_var_eq|, FALSE', ' == TRUE'];
            this.left = new typeconvert_1.CAsUniversalVar(scope, this.left, leftType);
            this.right = new typeconvert_1.CAsUniversalVar(scope, this.right, rightType);
            scope.root.headerFlags.js_var_eq = true;
        }
        else if (leftType instanceof types_1.StructType || leftType instanceof types_1.ArrayType || leftType instanceof types_1.DictType
            || rightType instanceof types_1.StructType || rightType instanceof types_1.ArrayType || rightType instanceof types_1.DictType) {
            operatorMap[ts.SyntaxKind.ExclamationEqualsEqualsToken] = '!=';
            operatorMap[ts.SyntaxKind.ExclamationEqualsToken] = '!=';
            operatorMap[ts.SyntaxKind.EqualsEqualsEqualsToken] = '==';
            operatorMap[ts.SyntaxKind.EqualsEqualsToken] = '==';
            if (isEqualityOp && leftType != rightType) {
                this.expression = "FALSE";
                scope.root.headerFlags.bool = true;
            }
        }
        this.operator = operatorMap[operatorKind];
        if (callReplaceMap[operatorKind]) {
            this.replacedWithCall = true;
            _a = callReplaceMap[operatorKind], this.call = _a[0], this.callCondition = _a[1];
            if (this.call.indexOf('|') > -1)
                _b = this.call.split('|'), this.call = _b[0], this.callAddArgs = _b[1];
        }
        this.nodeText = node.flags & ts.NodeFlags.Synthesized ? "(synthesized node)" : node.getText();
        var _a, _b;
    }
    CBinaryExpression = __decorate([
        template_1.CodeTemplate("\n{#if expression}\n    {expression}\n{#elseif operator}\n    {left} {operator} {right}\n{#elseif replacedWithCall}\n    {call}({left}, {right}{callAddArgs}){callCondition}\n{#else}\n    /* unsupported expression {nodeText} */\n{/if}", ts.SyntaxKind.BinaryExpression)
    ], CBinaryExpression);
    return CBinaryExpression;
}());
exports.CBinaryExpression = CBinaryExpression;
var CArithmeticExpression = /** @class */ (function () {
    function CArithmeticExpression(scope, node) {
        this.operator = null;
        this.computeOperation = null;
        var leftType = scope.root.typeHelper.getCType(node.left);
        var rightType = scope.root.typeHelper.getCType(node.right);
        this.isCompoundAssignment = typeguards_1.isCompoundAssignment(node.operatorToken);
        if (types_1.toNumberCanBeNaN(leftType) || types_1.toNumberCanBeNaN(rightType)) {
            var js_var_operator_map = (_a = {},
                _a[ts.SyntaxKind.AsteriskToken] = "JS_VAR_ASTERISK",
                _a[ts.SyntaxKind.AsteriskEqualsToken] = "JS_VAR_ASTERISK",
                _a[ts.SyntaxKind.SlashToken] = "JS_VAR_SLASH",
                _a[ts.SyntaxKind.SlashEqualsToken] = "JS_VAR_SLASH",
                _a[ts.SyntaxKind.PercentToken] = "JS_VAR_PERCENT",
                _a[ts.SyntaxKind.PercentEqualsToken] = "JS_VAR_PERCENT",
                _a[ts.SyntaxKind.MinusToken] = "JS_VAR_MINUS",
                _a[ts.SyntaxKind.MinusEqualsToken] = "JS_VAR_MINUS",
                _a[ts.SyntaxKind.LessThanLessThanToken] = "JS_VAR_SHL",
                _a[ts.SyntaxKind.LessThanLessThanEqualsToken] = "JS_VAR_SHL",
                _a[ts.SyntaxKind.GreaterThanGreaterThanToken] = "JS_VAR_SHR",
                _a[ts.SyntaxKind.GreaterThanGreaterThanEqualsToken] = "JS_VAR_SHR",
                _a[ts.SyntaxKind.GreaterThanGreaterThanGreaterThanToken] = "JS_VAR_USHR",
                _a[ts.SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken] = "JS_VAR_USHR",
                _a[ts.SyntaxKind.BarToken] = "JS_VAR_OR",
                _a[ts.SyntaxKind.BarEqualsToken] = "JS_VAR_OR",
                _a[ts.SyntaxKind.AmpersandToken] = "JS_VAR_AND",
                _a[ts.SyntaxKind.AmpersandEqualsToken] = "JS_VAR_AND",
                _a);
            this.computeOperation = js_var_operator_map[node.operatorToken.kind];
            this.left = new typeconvert_1.CAsUniversalVar(scope, node.left);
            this.right = new typeconvert_1.CAsUniversalVar(scope, node.right);
            scope.root.headerFlags.js_var_compute = true;
        }
        else {
            this.operator = node.operatorToken.getText();
            this.left = new typeconvert_1.CAsNumber(scope, node.left);
            this.right = new typeconvert_1.CAsNumber(scope, node.right);
            if (node.operatorToken.kind == ts.SyntaxKind.GreaterThanGreaterThanGreaterThanToken
                || node.operatorToken.kind == ts.SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken) {
                this.operator = ">>";
                var leftAsString = template_1.CodeTemplateFactory.templateToString(this.left);
                this.left = "((uint16_t)" + leftAsString + ")";
                if (node.operatorToken.kind == ts.SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken)
                    this.left = leftAsString + " = " + this.left;
                scope.root.headerFlags.uint16_t = true;
            }
        }
        this.nodeText = node.flags & ts.NodeFlags.Synthesized ? "(synthesized node)" : node.getText();
        var _a;
    }
    CArithmeticExpression = __decorate([
        template_1.CodeTemplate("\n{#if operator}\n    {left} {operator} {right}\n{#elseif computeOperation && isCompoundAssignment}\n    {left} = js_var_compute({left}, {computeOperation}, {right})\n{#elseif computeOperation}\n    js_var_compute({left}, {computeOperation}, {right})\n{#else}\n    /* unsupported arithmetic expression {nodeText} */\n{/if}")
    ], CArithmeticExpression);
    return CArithmeticExpression;
}());
var CPlusExpression = /** @class */ (function () {
    function CPlusExpression(scope, node) {
        this.addNumbers = false;
        this.isUniversalVar = false;
        this.replacedWithVar = false;
        this.replacementVarName = null;
        this.gcVarName = null;
        var leftType = scope.root.typeHelper.getCType(node.left);
        this.left = template_1.CodeTemplateFactory.createForNode(scope, node.left);
        var rightType = scope.root.typeHelper.getCType(node.right);
        this.right = template_1.CodeTemplateFactory.createForNode(scope, node.right);
        if (leftType == types_1.RegexVarType) {
            leftType = types_1.StringVarType;
            this.left = new regexfunc_1.CRegexAsString(this.left);
        }
        if (rightType == types_1.RegexVarType) {
            rightType = types_1.StringVarType;
            this.right = new regexfunc_1.CRegexAsString(this.right);
        }
        if ((leftType === types_1.NumberVarType || leftType === types_1.BooleanVarType) && (rightType === types_1.NumberVarType || rightType === types_1.BooleanVarType)) {
            this.addNumbers = true;
        }
        else if (leftType === types_1.UniversalVarType || rightType === types_1.UniversalVarType) {
            this.isUniversalVar = true;
            this.left = new typeconvert_1.CAsUniversalVar(scope, this.left, leftType);
            this.right = new typeconvert_1.CAsUniversalVar(scope, this.right, rightType);
            scope.root.headerFlags.js_var_plus = true;
        }
        else {
            var tempVarName = scope.root.memoryManager.getReservedTemporaryVarName(node);
            scope.func.variables.push(new variable_1.CVariable(scope, tempVarName, "char *", { initializer: "NULL" }));
            this.gcVarName = scope.root.memoryManager.getGCVariableForNode(node);
            this.replacedWithVar = true;
            this.replacementVarName = tempVarName;
            this.strlen_left = new typeconvert_1.CAsString_Length(scope, node.left, this.left, leftType);
            this.strlen_right = new typeconvert_1.CAsString_Length(scope, node.right, this.right, rightType);
            this.strcat_left = new typeconvert_1.CAsString_Concat(scope, node.left, tempVarName, this.left, leftType);
            this.strcat_right = new typeconvert_1.CAsString_Concat(scope, node.right, tempVarName, this.right, rightType);
            scope.root.headerFlags.strings = true;
            scope.root.headerFlags.malloc = true;
            scope.root.headerFlags.str_int16_t_cat = true;
            if (this.gcVarName) {
                scope.root.headerFlags.gc_iterator = true;
                scope.root.headerFlags.array = true;
            }
        }
    }
    CPlusExpression = __decorate([
        template_1.CodeTemplate("\n{#statements}\n    {#if replacedWithVar}\n        {replacementVarName} = malloc({strlen_left} + {strlen_right} + 1);\n        assert({replacementVarName} != NULL);\n        {replacementVarName}[0] = '\\0';\n        {strcat_left}\n        {strcat_right}\n    {/if}\n    {#if replacedWithVar && gcVarName}\n        ARRAY_PUSH({gcVarName}, {replacementVarName});\n    {/if}\n\n{/statements}\n{#if addNumbers}\n    {left} + {right}\n{#elseif replacedWithVar}\n    {replacementVarName}\n{#elseif isUniversalVar}\n    js_var_plus({left}, {right})\n{/if}")
    ], CPlusExpression);
    return CPlusExpression;
}());
var CUnaryExpression = /** @class */ (function () {
    function CUnaryExpression(scope, node) {
        this.call = null;
        this.expression = null;
        var operatorMap = {};
        var type = scope.root.typeHelper.getCType(node.operand);
        this.operand = template_1.CodeTemplateFactory.createForNode(scope, node.operand);
        this.isPostfix = node.kind == ts.SyntaxKind.PostfixUnaryExpression;
        this.nodeText = node.getText();
        operatorMap[ts.SyntaxKind.ExclamationToken] = '!';
        if (type == types_1.NumberVarType || type == types_1.BooleanVarType) {
            operatorMap[ts.SyntaxKind.PlusPlusToken] = '++';
            operatorMap[ts.SyntaxKind.MinusMinusToken] = '--';
            operatorMap[ts.SyntaxKind.MinusToken] = '-';
            operatorMap[ts.SyntaxKind.PlusToken] = '+';
            operatorMap[ts.SyntaxKind.TildeToken] = '~';
        }
        if (node.operator === ts.SyntaxKind.PlusToken) {
            if (type == types_1.StringVarType) {
                this.call = "str_to_int16_t";
                scope.root.headerFlags.str_to_int16_t = true;
            }
            else if (type == types_1.UniversalVarType) {
                this.call = "js_var_to_number";
                scope.root.headerFlags.js_var_to_number = true;
            }
            else if (type instanceof types_1.StructType) {
                this.call = "js_var_from";
                this.operand = "JS_VAR_NAN";
                scope.root.headerFlags.js_var_from = true;
            }
            else if (type instanceof types_1.ArrayType && !type.isDynamicArray && type.capacity == 0) {
                this.expression = "js_var_from_int16_t(0)";
                scope.root.headerFlags.js_var_from_int16_t = true;
            }
            else if (type instanceof types_1.ArrayType && !type.isDynamicArray && type.capacity > 1) {
                this.expression = "js_var_from(JS_VAR_NAN)";
                scope.root.headerFlags.js_var_from = true;
            }
            else if (type instanceof types_1.DictType) {
                this.expression = "js_var_from(JS_VAR_NAN)";
                scope.root.headerFlags.js_var_from = true;
            }
        }
        this.operator = operatorMap[node.operator];
    }
    CUnaryExpression = __decorate([
        template_1.CodeTemplate("\n{#if isPostfix && operator}\n    {operand}{operator}\n{#elseif !isPostfix && operator}\n    {operator}{operand}\n{#elseif call}\n    {call}({operand})\n{#elseif expression}\n    {expression}\n{#else}\n    /* unsupported expression {nodeText} */\n{/if}", [ts.SyntaxKind.PrefixUnaryExpression, ts.SyntaxKind.PostfixUnaryExpression])
    ], CUnaryExpression);
    return CUnaryExpression;
}());
var CTernaryExpression = /** @class */ (function () {
    function CTernaryExpression(scope, node) {
        this.condition = template_1.CodeTemplateFactory.createForNode(scope, node.condition);
        this.whenTrue = template_1.CodeTemplateFactory.createForNode(scope, node.whenTrue);
        this.whenFalse = template_1.CodeTemplateFactory.createForNode(scope, node.whenFalse);
    }
    CTernaryExpression = __decorate([
        template_1.CodeTemplate("{condition} ? {whenTrue} : {whenFalse}", ts.SyntaxKind.ConditionalExpression)
    ], CTernaryExpression);
    return CTernaryExpression;
}());
var CGroupingExpression = /** @class */ (function () {
    function CGroupingExpression(scope, node) {
        this.expression = template_1.CodeTemplateFactory.createForNode(scope, node.expression);
    }
    CGroupingExpression = __decorate([
        template_1.CodeTemplate("({expression})", ts.SyntaxKind.ParenthesizedExpression)
    ], CGroupingExpression);
    return CGroupingExpression;
}());
var CTypeOf = /** @class */ (function () {
    function CTypeOf(scope, node) {
        var type = scope.root.typeHelper.getCType(node.expression);
        this.isUniversalVar = type === types_1.UniversalVarType;
        this.isString = type === types_1.StringVarType;
        this.isNumber = type === types_1.NumberVarType;
        this.isBoolean = type === types_1.BooleanVarType;
        this.expression = template_1.CodeTemplateFactory.createForNode(scope, node.expression);
        if (type == types_1.UniversalVarType) {
            scope.root.headerFlags.js_var = true;
            scope.root.headerFlags.js_var_typeof = true;
        }
    }
    CTypeOf = __decorate([
        template_1.CodeTemplate("\n{#if isUniversalVar}\n    js_var_typeof({expression})\n{#elseif isString}\n    \"string\"\n{#elseif isNumber}\n    \"number\"\n{#elseif isBoolean}\n    \"number\"\n{#else}\n    \"object\"\n{/if}", ts.SyntaxKind.TypeOfExpression)
    ], CTypeOf);
    return CTypeOf;
}());
var CVoid = /** @class */ (function () {
    function CVoid(scope, node) {
        this.expression = template_1.CodeTemplateFactory.createForNode(scope, node.expression);
        scope.root.headerFlags.js_var = true;
        scope.root.headerFlags.js_var_to_undefined = true;
    }
    CVoid = __decorate([
        template_1.CodeTemplate("js_var_to_undefined({expression})", ts.SyntaxKind.VoidExpression)
    ], CVoid);
    return CVoid;
}());
var CDelete = /** @class */ (function () {
    function CDelete(scope, node) {
        this.topExpressionOfStatement = node.parent.kind == ts.SyntaxKind.ExpressionStatement;
        this.dict = (ts.isPropertyAccessExpression(node.expression) || ts.isElementAccessExpression(node.expression))
            && template_1.CodeTemplateFactory.createForNode(scope, node.expression.expression);
        if (ts.isElementAccessExpression(node.expression))
            this.argExpression = ts.isNumericLiteral(node.expression.argumentExpression)
                ? '"' + node.expression.argumentExpression.text + '"'
                : template_1.CodeTemplateFactory.createForNode(scope, node.expression.argumentExpression);
        else if (ts.isPropertyAccessExpression(node.expression))
            this.argExpression = new literals_1.CString(scope, node.expression.name.text);
        this.tempVarName = scope.root.symbolsHelper.addTemp(node, "tmp_dict_pos");
        scope.variables.push(new variable_1.CVariable(scope, this.tempVarName, types_1.NumberVarType));
        scope.root.headerFlags.bool = true;
        scope.root.headerFlags.array_remove = true;
    }
    CDelete = __decorate([
        template_1.CodeTemplate("\n{#statements}\n    {tempVarName} = dict_find_pos({dict}->index->data, {dict}->index->size, {argExpression});\n    if ({tempVarName} >= 0)\n    {\n        ARRAY_REMOVE({dict}->index, {tempVarName}, 1);\n        ARRAY_REMOVE({dict}->values, {tempVarName}, 1);\n    }\n{/statements}\n{#if !topExpressionOfStatement}\n    TRUE\n{/if}", ts.SyntaxKind.DeleteExpression)
    ], CDelete);
    return CDelete;
}());
//# sourceMappingURL=expressions.js.map