"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
function isNode(n) {
    return n && n.kind !== undefined && n.flags !== undefined && n.pos !== undefined && n.end !== undefined;
}
exports.isNode = isNode;
function isEqualsExpression(n) {
    return n && n.kind == ts.SyntaxKind.BinaryExpression && n.operatorToken.kind == ts.SyntaxKind.EqualsToken;
}
exports.isEqualsExpression = isEqualsExpression;
function isMethodCall(n) {
    return ts.isCallExpression(n) && ts.isPropertyAccessExpression(n.expression);
}
exports.isMethodCall = isMethodCall;
function isFunctionArgInMethodCall(n) {
    return ts.isFunctionExpression(n) && ts.isCallExpression(n.parent) && n.parent.arguments[0] == n && ts.isPropertyAccessExpression(n.parent.expression);
}
exports.isFunctionArgInMethodCall = isFunctionArgInMethodCall;
function isFieldElementAccess(n) {
    return ts.isElementAccessExpression(n) && (!ts.isCallExpression(n.parent) || n.parent.expression != n);
}
exports.isFieldElementAccess = isFieldElementAccess;
function isFieldPropertyAccess(n) {
    return ts.isPropertyAccessExpression(n) && (!ts.isCallExpression(n.parent) || n.parent.expression != n);
}
exports.isFieldPropertyAccess = isFieldPropertyAccess;
function isForOfWithSimpleInitializer(n) {
    return ts.isForOfStatement(n) && ts.isVariableDeclarationList(n.initializer) && n.initializer.declarations.length == 1;
}
exports.isForOfWithSimpleInitializer = isForOfWithSimpleInitializer;
function isForOfWithIdentifierInitializer(n) {
    return ts.isForOfStatement(n) && ts.isIdentifier(n.initializer);
}
exports.isForOfWithIdentifierInitializer = isForOfWithIdentifierInitializer;
function isLiteral(n) {
    return ts.isNumericLiteral(n) || ts.isStringLiteral(n) || ts.isRegularExpressionLiteral(n) || n.kind == ts.SyntaxKind.TrueKeyword || n.kind == ts.SyntaxKind.FalseKeyword;
}
exports.isLiteral = isLiteral;
function isConvertToNumberExpression(n) {
    return ts.isPrefixUnaryExpression(n) && n.operator === ts.SyntaxKind.PlusToken;
}
exports.isConvertToNumberExpression = isConvertToNumberExpression;
exports.SyntaxKind_NaNKeyword = ts.SyntaxKind.Count + 1;
function isNullOrUndefinedOrNaN(n) {
    return n.kind === ts.SyntaxKind.NullKeyword || n.kind === ts.SyntaxKind.UndefinedKeyword || n.kind === exports.SyntaxKind_NaNKeyword;
}
exports.isNullOrUndefinedOrNaN = isNullOrUndefinedOrNaN;
function isDeleteExpression(n) {
    return ts.isDeleteExpression(n) && (ts.isPropertyAccessExpression(n.expression) || ts.isElementAccessExpression(n.expression));
}
exports.isDeleteExpression = isDeleteExpression;
function isThisKeyword(n) {
    return n.kind === ts.SyntaxKind.ThisKeyword;
}
exports.isThisKeyword = isThisKeyword;
function isCompoundAssignment(n) {
    return n.kind >= ts.SyntaxKind.FirstCompoundAssignment && n.kind <= ts.SyntaxKind.LastCompoundAssignment;
}
exports.isCompoundAssignment = isCompoundAssignment;
var toNumberOps = [
    ts.SyntaxKind.MinusToken, ts.SyntaxKind.MinusEqualsToken,
    ts.SyntaxKind.AsteriskToken, ts.SyntaxKind.AsteriskEqualsToken,
    ts.SyntaxKind.SlashToken, ts.SyntaxKind.SlashEqualsToken,
    ts.SyntaxKind.PercentToken, ts.SyntaxKind.PercentEqualsToken,
];
var toIntegerOps = [
    ts.SyntaxKind.LessThanLessThanToken, ts.SyntaxKind.LessThanLessThanEqualsToken,
    ts.SyntaxKind.GreaterThanGreaterThanToken, ts.SyntaxKind.GreaterThanGreaterThanEqualsToken,
    ts.SyntaxKind.GreaterThanGreaterThanGreaterThanToken, ts.SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken,
    ts.SyntaxKind.BarToken, ts.SyntaxKind.BarEqualsToken,
    ts.SyntaxKind.AmpersandToken, ts.SyntaxKind.AmpersandEqualsToken
];
function isNumberOp(op) {
    return toNumberOps.indexOf(op) > -1;
}
exports.isNumberOp = isNumberOp;
function isIntegerOp(op) {
    return toIntegerOps.indexOf(op) > -1;
}
exports.isIntegerOp = isIntegerOp;
//# sourceMappingURL=typeguards.js.map